@@ -4,16 +4,21 @@ import os
 
 import jwt
 import requests
-from stream.serializer import _datetime_encoder
+from requests import Request
 
 from stream import exceptions, serializer
-from stream.users import Users
-from stream.utils import validate_feed_slug, validate_user_id, validate_foreign_id_time
-from requests import Request
-from stream.reactions import Reactions
 from stream.collections import Collections
-from stream.personalization import Personalization
 from stream.feed import Feed
+from stream.personalization import Personalization
+from stream.reactions import Reactions
+from stream.serializer import _datetime_encoder
+from stream.users import Users
+from stream.utils import (
+    validate_feed_slug,
+    validate_foreign_id_time,
+    validate_user_id,
+    get_reaction_params,
+)
 
 try:
     from urllib.parse import urlparse
@@ -23,7 +28,7 @@ except ImportError:
 logger = logging.getLogger(__name__)
 
 
-class StreamClient(object):
+class StreamClient:
     def __init__(
         self,
         api_key,
@@ -169,7 +174,8 @@ class StreamClient(object):
         return parsed_result
 
     def create_user_token(self, user_id, **extra_data):
-        """Setup the payload for the given user_id with optional
+        """
+        Setup the payload for the given user_id with optional
         extra data (key, value pairs) and encode it using jwt
         """
         payload = {"user_id": user_id}
@@ -177,15 +183,12 @@ class StreamClient(object):
             payload[k] = v
         return jwt.encode(payload, self.api_secret, algorithm="HS256").decode("utf-8")
 
-    def create_user_session_token(self, user_id, **extra_data):
-        return self.create_user_token(user_id, **extra_data)
-
-    def create_jwt_token(self, resource, action, feed_id=None, user_id=None):
+    def create_jwt_token(self, resource, action, feed_id=None, user_id=None, **params):
         """
         Setup the payload for the given resource, action, feed or user
         and encode it using jwt
         """
-        payload = {"action": action, "resource": resource}
+        payload = {**params, "action": action, "resource": resource}
         if feed_id is not None:
             payload["feed_id"] = feed_id
         if user_id is not None:
@@ -379,9 +382,7 @@ class StreamClient(object):
         if enrich or reactions is not None:
             endpoint = "enrich/" + endpoint
 
-        query_params = {}
-        for key in params:
-            query_params[key] = params[key]
+        query_params = {**params}
 
         if ids is not None:
             query_params["ids"] = ",".join(ids)
@@ -393,21 +394,12 @@ class StreamClient(object):
             query_params["foreign_ids"] = ",".join(foreign_ids)
             query_params["timestamps"] = ",".join(timestamps)
 
-        if reactions is not None and not isinstance(reactions, (dict,)):
-            raise TypeError("reactions argument should be a dictionary")
-
-        if reactions is not None:
-            if reactions.get("own"):
-                query_params["withOwnReactions"] = True
-            if reactions.get("recent"):
-                query_params["withRecentReactions"] = True
-            if reactions.get("counts"):
-                query_params["withReactionCounts"] = True
+        query_params.update(get_reaction_params(reactions))
 
         return self.get(endpoint, auth_token, params=query_params)
 
     def activity_partial_update(
-        self, id=None, foreign_id=None, time=None, set={}, unset=[]
+        self, id=None, foreign_id=None, time=None, set=None, unset=None
     ):
         """
         Partial update activity, via activity ID or Foreign ID + timestamp
@@ -428,7 +420,7 @@ class StreamClient(object):
                 "Only one of the id or the foreign_id+time parameters can be provided"
             )
 
-        data = {"set": set, "unset": unset}
+        data = {"set": set or {}, "unset": unset or []}
 
         if id is not None:
             data["id"] = id
@@ -438,7 +430,7 @@ class StreamClient(object):
 
         return self.activities_partial_update(updates=[data])
 
-    def activities_partial_update(self, updates=[]):
+    def activities_partial_update(self, updates=None):
         """
         Partial update activity, via activity ID or Foreign ID + timestamp
 
@@ -462,7 +454,7 @@ class StreamClient(object):
 
         auth_token = self.create_jwt_token("activities", "*", feed_id="*")
 
-        data = {"changes": updates}
+        data = {"changes": updates or []}
 
         return self.post("activity/", auth_token, data=data)
 
